use core::arch::asm;

use crate::error::{Error, Result};
use raw_cpuid::CpuId;
use x86_64::registers::control::{Cr0, Cr0Flags};

pub fn enable_fpu() -> Result<()> {
    let cpuid = CpuId::new();

    if let Some(features) = cpuid.get_feature_info() {
        if features.has_fpu() && features.has_sse() && features.has_sse2() {
            let mut cr0 = Cr0::read();

            cr0.set(Cr0Flags::MONITOR_COPROCESSOR, true);
            cr0.set(Cr0Flags::EXTENSION_TYPE, true);

            cr0.set(Cr0Flags::EMULATE_COPROCESSOR, false);
            cr0.set(Cr0Flags::TASK_SWITCHED, false);

            unsafe {
                Cr0::write(cr0);
                asm!("fninit");
            }

            log::info!("FPU enabled");
            Ok(())
        } else {
            Err(Error::NoSSE)
        }
    } else {
        Err(Error::NoCpuIdFeatures)
    }
}
