use ucs2::decode::traits::DecodeUCS2Char;
use uefi::{
    proto::console::text::{Input as Stdin, Key, ScanCode},
    table::{Boot, SystemTable},
    Char16, Event,
};

pub enum CharOrScanCode {
    Char(char),
    ScanCode(ScanCode),
}

pub struct Input {
    st: SystemTable<Boot>,
    event: [Event; 1],
}

impl Input {
    pub fn new(st: &SystemTable<Boot>) -> Self {
        let (st, event) = unsafe {
            let mut st = st.unsafe_clone();
            let event = [st.stdin().wait_for_key_event().unsafe_clone()];
            (st, event)
        };

        Self { st, event }
    }

    fn stdin(&mut self) -> &mut Stdin {
        self.st.stdin()
    }

    fn decode_char16(ch: Char16) -> Option<char> {
        DecodeUCS2Char::decode_ucs2(u16::from(ch)).into_char()
    }

    pub fn next_key(&mut self) -> Option<CharOrScanCode> {
        let ref mut events = self.event;

        self.st.boot_services().wait_for_event(events).ok()?;

        if let Some(key) = self.stdin().read_key().ok()? {
            match key {
                Key::Printable(ch) => Some(CharOrScanCode::Char(Self::decode_char16(ch)?)),
                Key::Special(ch) => Some(CharOrScanCode::ScanCode(ch)),
            }
        } else {
            None
        }
    }
}
