use core::fmt::Debug;

use alloc::string::String;
use alloc::vec::Vec;

use crate::structs::KnownObjectId;

#[derive(Clone)]
pub struct INode {
    pub id: u64,
    pub(crate) path: Vec<Vec<u8>>,
}

pub struct PathDebug<'a>(&'a Vec<Vec<u8>>);

impl<'a> Debug for PathDebug<'a> {
    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
        write!(f, "\"")?;
        for segment in self.0 {
            write!(f, "/{}", String::from_utf8_lossy(&segment))?;
        }
        write!(f, "\"")?;
        Ok(())
    }
}

impl core::fmt::Debug for INode {
    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
        f.debug_struct("INode")
            .field("id", &self.id)
            .field("path", &PathDebug(&self.path))
            .finish()
    }
}

impl INode {
    pub fn id(&self) -> KnownObjectId {
        self.id.into()
    }

    pub fn into_child(mut self, id: u64, path: Vec<u8>) -> Self {
        self.path.push(path);
        Self { id, ..self }
    }
}
